@@ -1,7 +1,7 @@
 
 /*
  *  agUtils.c
- *  $Id: agUtils.c,v 4.8 2006/03/25 19:24:49 bkorb Exp $
+ *  $Id: agUtils.c,v 4.8 2006/08/01 20:31:04 bkorb Exp $
  *  This is the main routine for autogen.
  */
 
@@ -81,6 +81,35 @@ aprf( const char* pzFmt, ... )
 }
 
 
+LOCAL char*
+mkstempPat( void )
+{
+    static char const * pzTmpDir = NULL;
+
+    if (pzTmpDir == NULL) {
+        static char const * const envlist[] = {
+            "TMPDIR", "TEMP", "TMP", NULL };
+        char const * const * penv = envlist;
+        for (;;) {
+            pzTmpDir = getenv(*penv);
+            if (pzTmpDir != NULL)
+                break;
+            if (*(++penv) == NULL) {
+                pzTmpDir = "/tmp";
+                break;
+            }
+        }
+    }
+
+    {
+        char* pzRes = aprf("%s/agtmp.XXXXXX", pzTmpDir);
+        manageAllocatedData((void*)pzRes);
+        TAGMEM( pzRes, "temp file template" );
+        return pzRes;
+    }
+}
+
+
 LOCAL void
 doOptions( int arg_ct, char** arg_vec )
 {
