@@ -1,12 +1,12 @@
 
 /*
- *  $Id: defLex.c,v 4.11 2006/01/24 23:22:16 bkorb Exp $
+ *  $Id: defLex.c,v 4.13 2006/06/24 23:36:07 bkorb Exp $
  *  This module scans the template variable declarations and passes
  *  tokens back to the parser.
  */
 
 /*
- *  AutoGen copyright 1992-2005 Bruce Korb
+ *  AutoGen copyright 1992-2006 Bruce Korb
  *
  *  AutoGen is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -353,7 +353,7 @@ loadScheme( void )
     *pzEnd = endCh;
 
     pCurCtx->pzScan = pzEnd;
-    pzEnd = resolveSCM( res );
+    pzEnd = (char*)resolveSCM( res ); /* ignore const-ness */
     if (strlen( pzEnd ) >= schemeLen) {
         AGDUPSTR( pzEnd, pzEnd, "SCM Result" );
 
@@ -386,7 +386,7 @@ alist_to_autogen_def( void )
     char*  pzEnd   = (char*)skipScheme( pzText, pzText + strlen( pzText ));
 
     SCM    res;
-    int    res_len;
+    u_int  res_len;
     tScanCtx*  pCtx;
 
     /*
@@ -568,7 +568,7 @@ assembleHereString( char* pzScan )
 {
     ag_bool  trimTabs = AG_FALSE;
     char     zMark[ MAX_HEREMARK_LEN ];
-    int      markLen = 0;
+    u_int    markLen = 0;
     char*    pzDest;
 
     /*
