@@ -1,6 +1,6 @@
 
 /*
- *  $Id: expOutput.c,v 4.9 2006/03/25 19:24:49 bkorb Exp $
+ *  $Id: expOutput.c,v 4.15 2006/08/01 20:31:04 bkorb Exp $
  *
  *  This module implements the output file manipulation function
  */
@@ -67,14 +67,14 @@ removeWriteAccess( int fd )
      *  Set our usage mask to all all the access
      *  bits that do not provide for write access
      */
-#   define USE_MASK ~(S_IWUSR|S_IWGRP|S_IWOTH)
+#   define USE_MASK ((unsigned)(~(S_IWUSR|S_IWGRP|S_IWOTH)))
     fstat( fd, &sbuf );
 
     /*
      *  Mask off the write permission bits, but ensure that
      *  the user read bit is set.
      */
-    sbuf.st_mode = (sbuf.st_mode & USE_MASK) | S_IRUSR;
+    sbuf.st_mode = ((unsigned)sbuf.st_mode & USE_MASK) | S_IRUSR;
     fchmod( fd, sbuf.st_mode & S_IAMB );
 }
 
@@ -174,7 +174,7 @@ ag_scm_out_pop( SCM ret_contents )
         AG_ABEND( "ERROR:  Cannot pop output with no output pushed\n" );
 
     if (AG_SCM_BOOL_P( ret_contents ) && SCM_NFALSEP( ret_contents )) {
-        long pos = ftell( pCurFp->pFile );
+        unsigned long pos = ftell( pCurFp->pFile );
 
         if (pos != 0) {
             char* pz = ag_scribble( pos );
@@ -234,7 +234,7 @@ ag_scm_out_suspend( SCM suspName )
 
 /*=gfunc out_resume
  *
- * what:   resume current output file
+ * what:   resume suspended output file
  * exparg: suspName, A name tag for reactivating
  * doc:  
  *  If there has been a suspended output, then make that output descriptor
@@ -266,6 +266,84 @@ ag_scm_out_resume( SCM suspName )
 }
 
 
+/*=gfunc ag_fprintf
+ *
+ * what:  format to autogen stream
+ * general_use:
+ *
+ * exparg: ag-diversion, AutoGen diversion name or number
+ * exparg: format,       formatting string
+ * exparg: format-arg,   list of arguments to formatting string, opt, list
+ *
+ * doc:  Format a string using arguments from the alist.
+ *       Write to a specified AutoGen diversion.
+ *       That may be either a specified suspended output stream
+ *       (@pxref{SCM out-suspend}) or an index into the output stack
+ *       (@pxref{SCM out-push-new}).  @code{(ag-fprintf 0 ...)} is
+ *       equivalent to @code{(emit (sprintf ...))}, and
+ *       @code{(ag-fprintf 1 ...)} sends output to the most recently
+ *       suspended output stream.
+=*/
+SCM
+ag_scm_ag_fprintf( SCM port, SCM fmt, SCM alist )
+{
+    static const char invalid_z[] = "ag-fprintf: 'port' is invalid";
+    int   list_len = scm_ilength( alist );
+    char* pzFmt    = ag_scm2zchars( fmt, zFormat );
+    SCM   res      = run_printf( pzFmt, list_len, alist );
+
+    /*
+     *  If "port" is a string, it must match one of the suspended outputs.
+     *  Otherwise, we'll fall through to the abend.
+     */
+    if (AG_SCM_STRING_P(port)) {
+        int  ix  = 0;
+        tCC* pzName = ag_scm2zchars( port, "resume name" );
+
+        for (; ix < suspendCt; ix++) {
+            if (strcmp( pSuspended[ ix ].pzSuspendName, pzName ) == 0) {
+                tFpStack* pSaveFp = pCurFp;
+                pCurFp = pSuspended[ ix ].pOutDesc;
+                (void) ag_scm_emit( res );
+                pCurFp = pSaveFp;
+                return SCM_UNDEFINED;
+            }
+        }
+    }
+
+    /*
+     *  If "port" is a number, it is an index into the output stack with
+     *  "0" (zero) representing the current output and "1" the last suspended
+     *  output.  If the number is out of range, we'll fall through to the abend.
+     */
+    else if (AG_SCM_NUMBER_P(port)) do {
+        tFpStack* pSaveFp = pCurFp;
+        unsigned long val = gh_scm2ulong(port);
+
+        for (; val > 0; val--) {
+            pCurFp = pCurFp->pPrev;
+            if (pCurFp == NULL) {
+                pCurFp = pSaveFp;
+                goto fprintf_woops;
+            }
+        }
+
+        (void) ag_scm_emit( res );
+        pCurFp  = pSaveFp;
+        return SCM_UNDEFINED;
+        
+    } while (0);
+
+    /*
+     *  Still here?  We have a bad "port" specification.
+     */
+ fprintf_woops:
+    AG_ABEND( invalid_z );
+    /* NOTREACHED */
+    return SCM_UNDEFINED;
+}
+
+
 /*=gfunc out_push_add
  *
  * what:   append output to file
@@ -344,33 +422,22 @@ ag_scm_out_push_new( SCM new_file )
         addWriteAccess( pzNewFile );
         p->pFile = fopen( pzNewFile, "a" FOPEN_BINARY_FLAG "+" );
     }
-#   if defined(ENABLE_FMEMOPEN)
-    else if (! HAVE_OPT( NO_FMEMOPEN ))
-#   else
     else
-#   endif
+#if defined(ENABLE_FMEMOPEN)
+    if (! HAVE_OPT( NO_FMEMOPEN ))
+#endif
     {
         /*
          *  IF we do not have fopencookie(3GNU) or funopen(3BSD), then this
          *  block is a pure "else" clause.  If we do have one of those, then
-         *  the block is executed when a file name is not s specified *and*
+         *  the block is executed when a file name is not specified *and*
          *  --no-fmemopen was *not* selected.
          */
-        tSCC* pzTemp = NULL;
+        static char const * pzTemp = NULL;
         int tmpfd;
 
-        if (pzTemp == NULL) {
-            char* pz = getenv( "TEMP" );
-            if (pz == NULL) {
-                pz = getenv( "TMP" );
-                if (pz == NULL)
-                    pz = "/tmp";
-            }
-
-            pzTemp = aprf( "%s/agtmp.XXXXXX", pz );
-            manageAllocatedData((void*)pzTemp);
-            TAGMEM( pzTemp, "Saved temp file template" );
-        }
+        if (pzTemp == NULL)
+            pzTemp = mkstempPat();
 
         AGDUPSTR( pzNewFile, pzTemp, "temp file name" );
         p->flags |= FPF_UNLINK;
@@ -381,19 +448,19 @@ ag_scm_out_push_new( SCM new_file )
         p->pFile  = fopen( pzNewFile, "w" FOPEN_BINARY_FLAG "+" );
         close( tmpfd );
     }
-#   ifdef ENABLE_FMEMOPEN
-    else {
+#if defined(ENABLE_FMEMOPEN)
+    else
+    {
         /*
          *  This block is a pure "else" clause that is only compiled if neither
          *  "fopencookie" nor "funopen" is available in the local C library.
-         *  Otherwise, anonymous output without --no-fmemopen being selected
-         *  will get us here.
+         *  We reach here if --no-fmemopen was *not* on the command line.
          */
         p->pFile  = ag_fmemopen( NULL, 0, "wb+" );
-        pzNewFile = "in-mem buffer";
+        pzNewFile = (char*)"in-mem buffer";
         p->flags |= FPF_STATIC_NM | FPF_NOUNLINK | FPF_NOCHMOD;
     }
-#   endif /* ENABLE_FMEMOPEN */
+#endif /* ENABLE_FMEMOPEN */
 
     if (p->pFile == NULL)
         AG_ABEND( aprf( zCannot, errno, "open for write", pzNewFile,
