@@ -1,13 +1,15 @@
 
 /*
  *  stack.c
- *  $Id: stack.c,v 4.2 2005/01/24 00:00:37 bkorb Exp $
+ *  $Id: stack.c,v 4.7 2006/07/14 04:31:38 bkorb Exp $
+ *  Time-stamp:      "2006-07-13 21:11:29 bkorb"
+ *
  *  This is a special option processing routine that will save the
  *  argument to an option in a FIFO queue.
  */
 
 /*
- *  Automated Options copyright 1992-2005 Bruce Korb
+ *  Automated Options copyright 1992-2006 Bruce Korb
  *
  *  Automated Options is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -22,8 +24,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with Automated Options.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  *
  * As a special exception, Bruce Korb gives permission for additional
  * uses of the text contained in his release of AutoOpts.
@@ -49,9 +51,11 @@
  * If you do not wish that, delete this exception notice.
  */
 
-#include REGEX_HEADER
+#ifdef WITH_LIBREGEX
+#  include REGEX_HEADER
+#endif
 
-/*=export_func  unstackOptArg
+/*=export_func  optionUnstackArg
  * private:
  *
  * what:  Remove option args from a stack
@@ -62,9 +66,9 @@
  *  Invoked for options that are equivalenced to stacked options.
 =*/
 void
-unstackOptArg( pOpts, pOptDesc )
-    tOptions*  pOpts;
-    tOptDesc*  pOptDesc;
+optionUnstackArg(
+    tOptions*  pOpts,
+    tOptDesc*  pOptDesc )
 {
     int       res;
 
@@ -80,6 +84,7 @@ unstackOptArg( pOpts, pOptDesc )
         return;
     }
 
+#ifdef WITH_LIBREGEX
     {
         regex_t   re;
         int       i, ct, dIdx;
@@ -128,7 +133,45 @@ unstackOptArg( pOpts, pOptDesc )
 
         regfree( &re );
     }
+#else  /* not WITH_LIBREGEX */
+    {
+        int i, ct, dIdx;
 
+        /*
+         *  search the list for the entry(s) to remove.  Entries that
+         *  are removed are *not* copied into the result.  The source
+         *  index is incremented every time.  The destination only when
+         *  we are keeping a define.
+         */
+        for (i = 0, dIdx = 0, ct = pAL->useCt; --ct >= 0; i++) {
+            tCC*      pzSrc = pAL->apzArgs[ i ];
+            char*     pzEq  = strchr( pzSrc, '=' );
+
+            if (pzEq != NULL)
+                *pzEq = NUL;
+
+            if (strcmp( pzSrc, pOptDesc->pzLastArg ) == 0) {
+                /*
+                 *  Remove this entry by reducing the in-use count
+                 *  and *not* putting the string pointer back into
+                 *  the list.
+                 */
+                pAL->useCt--;
+            } else {
+                if (pzEq != NULL)
+                    *pzEq = '=';
+
+                /*
+                 *  IF we have dropped an entry
+                 *  THEN we have to move the current one.
+                 */
+                if (dIdx != i)
+                    pAL->apzArgs[ dIdx ] = pzSrc;
+                dIdx++;
+            }
+        }
+    }
+#endif /* WITH_LIBREGEX */
     /*
      *  IF we have unstacked everything,
      *  THEN indicate that we don't have any of these options
@@ -143,48 +186,15 @@ unstackOptArg( pOpts, pOptDesc )
 }
 
 
-/*=export_func  stackOptArg
- * private:
- *
- * what:  put option args on a stack
- * arg:   + tOptions* + pOpts    + program options descriptor +
- * arg:   + tOptDesc* + pOptDesc + the descriptor for this arg +
- *
- * doc:
- *  Keep an entry-ordered list of option arguments.
-=*/
-void
-stackOptArg( pOpts, pOptDesc )
-    tOptions*  pOpts;
-    tOptDesc*  pOptDesc;
+/*
+ *  Put an entry into an argument list.  The first argument points to
+ *  a pointer to the argument list structure.  It gets passed around
+ *  as an opaque address.
+ */
+LOCAL void
+addArgListEntry( void** ppAL, void* entry )
 {
-    tArgList* pAL;
-    tCC* pzLast = pOptDesc->pzLastArg;
-
-    if (pOptDesc->optArgType == ARG_NONE)
-        return;
-
-    if (pOptDesc->optActualIndex != pOptDesc->optIndex)
-        pOptDesc = pOpts->pOptDesc + pOptDesc->optActualIndex;
-
-    /*
-     *  IF this is a negated ('+'-marked) option
-     *  THEN we unstack the argument
-     */
-    if (DISABLED_OPT( pOptDesc )) {
-        if (pOptDesc->optCookie != NULL) {
-            AGFREE( pOptDesc->optCookie );
-            pOptDesc->optCookie = NULL;
-        }
-        pOptDesc->fOptState &= OPTST_PERSISTENT;
-        pOptDesc->fOptState |= OPTST_DISABLED;
-        return;
-    }
-
-    pAL = (tArgList*)pOptDesc->optCookie;
-
-    if (pzLast == NULL)
-        return;
+    tArgList* pAL = *(void**)ppAL;
 
     /*
      *  IF we have never allocated one of these,
@@ -196,6 +206,7 @@ stackOptArg( pOpts, pOptDesc )
             return;
         pAL->useCt   = 0;
         pAL->allocCt = MIN_ARG_ALLOC_CT;
+        *ppAL = (void*)pAL;
     }
 
     /*
@@ -214,13 +225,35 @@ stackOptArg( pOpts, pOptDesc )
         pAL = (tArgList*)AGREALOC( (void*)pAL, sz, "expanded opt arg stack" );
         if (pAL == NULL)
             return;
+        *ppAL = (void*)pAL;
     }
 
     /*
      *  Insert the new argument into the list
      */
-    pAL->apzArgs[ (pAL->useCt)++ ] = pzLast;
-    pOptDesc->optCookie = (void*)pAL;
+    pAL->apzArgs[ (pAL->useCt)++ ] = entry;
+}
+
+
+/*=export_func  optionStackArg
+ * private:
+ *
+ * what:  put option args on a stack
+ * arg:   + tOptions* + pOpts    + program options descriptor +
+ * arg:   + tOptDesc* + pOptDesc + the descriptor for this arg +
+ *
+ * doc:
+ *  Keep an entry-ordered list of option arguments.
+=*/
+void
+optionStackArg(
+    tOptions*  pOpts,
+    tOptDesc*  pOD )
+{
+    if (pOD->pzLastArg == NULL)
+        return;
+
+    addArgListEntry( &(pOD->optCookie), (void*)pOD->pzLastArg );
 }
 /*
  * Local Variables:
