@@ -1,6 +1,7 @@
 
 /*
- *  $Id: streqvcmp.c,v 4.3 2005/01/24 00:00:37 bkorb Exp $
+ *  $Id: streqvcmp.c,v 4.7 2006/07/27 02:59:08 bkorb Exp $
+ * Time-stamp:      "2006-07-26 18:25:53 bkorb"
  *
  *  String Equivalence Comparison
  *
@@ -11,7 +12,7 @@
  */
 
 /*
- *  Automated Options copyright 1992-2005 Bruce Korb
+ *  Automated Options copyright 1992-2006 Bruce Korb
  *
  *  Automated Options is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -26,8 +27,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with Automated Options.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  *
  * As a special exception, Bruce Korb gives permission for additional
  * uses of the text contained in his release of AutoOpts.
@@ -102,9 +103,9 @@ static unsigned char charmap[] = {
  *
  * what: compare two strings with an equivalence mapping
  *
- * arg:  + tCC* + str1 + first string +
- * arg:  + tCC* + str2 + second string +
- * arg:  + int  + ct   + compare length +
+ * arg:  + const char* + str1 + first string +
+ * arg:  + const char* + str2 + second string +
+ * arg:  + int         + ct   + compare length +
  *
  * ret_type:  int
  * ret_desc:  the difference between two differing characters
@@ -187,11 +188,19 @@ streqvcmp( tCC* s1, tCC* s2 )
  * doc:
  *
  * Set the character mapping.  If the count (@code{ct}) is set to zero, then
- * the map is first cleared by setting all entries in the map to their index
+ * the map is cleared by setting all entries in the map to their index
  * value.  Otherwise, the "@code{From}" character is mapped to the "@code{To}"
  * character.  If @code{ct} is greater than 1, then @code{From} and @code{To}
  * are incremented and the process repeated until @code{ct} entries have been
- * set. This function name is mapped to option_streqvmap so as to not conflict
+ * set. For example,
+ * @example
+ *    streqvmap( 'a', 'A', 26 );
+ * @end example
+ * @noindent
+ * will alter the mapping so that all English lower case letters
+ * will map to upper case.
+ *
+ * This function name is mapped to option_streqvmap so as to not conflict
  * with the POSIX name space.
  *
  * err:  none.
@@ -264,9 +273,7 @@ strequate( const char* s )
  * err:  none.
 =*/
 void
-strtransform( d, s )
-    char*       d;
-    const char* s;
+strtransform( char* d, const char* s )
 {
     do  {
         *(d++) = (char)charmap[ (unsigned)*s ];
