@@ -1,10 +1,10 @@
 
 /*
  *  xml2ag.c
- *  $Id: xml2ag.c,v 4.6 2005/11/07 00:22:05 bkorb Exp $
+ *  $Id: xml2ag.c,v 4.6 2006/06/24 23:36:08 bkorb Exp $
  *  This is the main routine for xml2ag.
  *
- *  xml2ag copyright 2002-2005 Bruce Korb
+ *  xml2ag copyright 2002-2006 Bruce Korb
  *
  *  xml2ag is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -133,7 +133,7 @@ main( int argc, char** argv )
     else {
         size_t sz;
         char*  pz = loadFile( stdin, &sz );
-        pDoc = xmlParseMemory( pz, sz );
+        pDoc = xmlParseMemory( pz, (int)sz );
         fprintf( outFp, "/* Parsed from stdin */\n" );
     }
 
@@ -242,7 +242,7 @@ trim( const char* pzSrc, size_t* pSz )
     }
 
     if (dataLen <= strSize) {
-        size_t sz = (strSize + 0x1000) & ~0x0FFF;
+        size_t sz = (strSize + 0x1000) & ~0x0FFFUL;
         if (pzData == NULL)
              pzData = malloc( sz );
         else pzData = realloc( pzData, sz );
@@ -421,7 +421,7 @@ printNode( xmlNodePtr pNode )
             char* pz = strstr( pzTxt, "*/" );
             if (pz == NULL)
                 break;
-            fwrite( pzTxt, (pz - pzTxt) + 1, 1, outFp );
+            fwrite( pzTxt, (unsigned)((pz - pzTxt) + 1), 1, outFp );
             pzTxt = pz+1;
             fputc( ' ', outFp );
         }
